/* eslint-disable no-magic-numbers */
import { SIZE_OF_EMPTY_ARRAY, ZERO } from '@/constants/common';
import { bioEntityResponseFixture } from '@/models/fixtures/bioEntityContentsFixture';
import { ELEMENT_SEARCH_RESULT_MOCK_ALIAS } from '@/models/mocks/elementSearchResultMock';
import { apiPath } from '@/redux/apiPath';
import { searchFitBounds } from '@/services/pluginsManager/map/triggerSearch/searchFitBounds';
import { mockNetworkNewAPIResponse, mockNetworkResponse } from '@/utils/mockNetworkResponse';
import { getReduxStoreWithActionsListener } from '@/utils/testing/getReduxStoreActionsListener';
import { waitFor } from '@testing-library/react';
import { HttpStatusCode } from 'axios';
import { bioEntityFixture } from '@/models/fixtures/bioEntityFixture';
import { handleAliasResults } from './handleAliasResults';

jest.mock('../../../../../../services/pluginsManager/map/triggerSearch/searchFitBounds');

const mockedAxiosClient = mockNetworkNewAPIResponse();
const mockedAxiosOldClient = mockNetworkResponse();

const SEARCH_CONFIG_MOCK = {
  point: {
    x: 500,
    y: 700,
  },
  maxZoom: 9,
  zoom: 5,
  isResultDrawerOpen: false,
};

describe('handleAliasResults - util', () => {
  beforeEach(() => {
    jest.clearAllMocks();

    const bioEntityWithIdReaction = bioEntityResponseFixture.content
      .filter(c => Boolean(c.bioEntity.idReaction))
      ?.map(b => b.bioEntity || { id: ZERO });

    mockedAxiosOldClient
      .onGet(
        apiPath.getReactionsWithIds(
          bioEntityWithIdReaction.map(bioEntity => Number(`${bioEntity.id}`)),
        ),
      )
      .reply(HttpStatusCode.Ok, []);
  });
  describe('when matching bioEntity not found', () => {
    it('should clear bio entities and do not close drawer if result drawer is not open', async () => {
      mockedAxiosClient
        .onGet(
          apiPath.getElementById(
            ELEMENT_SEARCH_RESULT_MOCK_ALIAS.id,
            ELEMENT_SEARCH_RESULT_MOCK_ALIAS.modelId,
          ),
        )
        .reply(HttpStatusCode.Ok, {
          ...bioEntityFixture,
          idReaction: undefined,
        });

      const { store } = getReduxStoreWithActionsListener();
      const { dispatch } = store;

      await handleAliasResults(dispatch, ELEMENT_SEARCH_RESULT_MOCK_ALIAS, {
        ...SEARCH_CONFIG_MOCK,
        searchDistance: '10',
      })(ELEMENT_SEARCH_RESULT_MOCK_ALIAS);

      await waitFor(() => {
        const actions = store.getActions();
        expect(actions.length).toBeGreaterThan(SIZE_OF_EMPTY_ARRAY);

        const actionTypes = actions.map(action => action.type);

        expect(actionTypes).toEqual([
          'project/getMultiBioEntity/pending',
          'project/getBioEntityById/pending',
          'project/getCommentElement/pending',
          'project/getCommentElement/fulfilled',
          'entityNumber/addNumbersToEntityNumberData',
          'project/getBioEntityById/fulfilled',
          'entityNumber/addNumbersToEntityNumberData',
          'project/getMultiBioEntity/fulfilled',
          'bioEntityContents/clearBioEntities',
        ]);
      });
    });
    it('should clear bio entities and close drawer if result drawer is already open', async () => {
      mockedAxiosClient
        .onGet(
          apiPath.getElementById(
            ELEMENT_SEARCH_RESULT_MOCK_ALIAS.id,
            ELEMENT_SEARCH_RESULT_MOCK_ALIAS.modelId,
          ),
        )
        .reply(HttpStatusCode.Ok, {
          ...bioEntityFixture,
          idReaction: undefined,
        });
      const { store } = getReduxStoreWithActionsListener();
      const { dispatch } = store;

      await handleAliasResults(dispatch, ELEMENT_SEARCH_RESULT_MOCK_ALIAS, {
        ...SEARCH_CONFIG_MOCK,
        searchDistance: '10',
        isResultDrawerOpen: true,
      })(ELEMENT_SEARCH_RESULT_MOCK_ALIAS);

      await waitFor(() => {
        const actions = store.getActions();
        expect(actions.length).toBeGreaterThan(SIZE_OF_EMPTY_ARRAY);

        const actionTypes = actions.map(action => action.type);

        expect(actionTypes).toEqual([
          'project/getMultiBioEntity/pending',
          'project/getBioEntityById/pending',
          'project/getCommentElement/pending',
          'project/getCommentElement/fulfilled',
          'entityNumber/addNumbersToEntityNumberData',
          'project/getBioEntityById/fulfilled',
          'entityNumber/addNumbersToEntityNumberData',
          'project/getMultiBioEntity/fulfilled',
          'drawer/closeDrawer',
          'bioEntityContents/clearBioEntities',
        ]);
      });
    });
  });
  describe('when matching bioEntity found', () => {
    it('should select tab and open bio entity drawer', async () => {
      mockedAxiosClient
        .onGet(
          apiPath.getElementById(
            ELEMENT_SEARCH_RESULT_MOCK_ALIAS.id,
            ELEMENT_SEARCH_RESULT_MOCK_ALIAS.modelId,
          ),
        )
        .reply(HttpStatusCode.Ok, {
          ...bioEntityFixture,
          x: 500,
          y: 700,
          width: 50,
          height: 50,
          idReaction: undefined,
        });

      const { store } = getReduxStoreWithActionsListener();
      const { dispatch } = store;

      await handleAliasResults(
        dispatch,
        ELEMENT_SEARCH_RESULT_MOCK_ALIAS,
        SEARCH_CONFIG_MOCK,
      )(ELEMENT_SEARCH_RESULT_MOCK_ALIAS);

      await waitFor(() => {
        const actions = store.getActions();
        expect(actions.length).toBeGreaterThan(SIZE_OF_EMPTY_ARRAY);

        const actionTypes = actions.map(action => action.type);

        expect(actionTypes).toEqual([
          'project/getMultiBioEntity/pending',
          'project/getBioEntityById/pending',
          'project/getCommentElement/pending',
          'project/getCommentElement/fulfilled',
          'entityNumber/addNumbersToEntityNumberData',
          'project/getBioEntityById/fulfilled',
          'entityNumber/addNumbersToEntityNumberData',
          'project/getMultiBioEntity/fulfilled',
          'drawer/selectTab',
          'drawer/openBioEntityDrawerById',
        ]);
      });
    });
  });

  describe('when searchDistance is not provided', () => {
    it('should select tab and open drawer without clearing bio entities', async () => {
      mockedAxiosClient
        .onGet(
          apiPath.getElementById(
            ELEMENT_SEARCH_RESULT_MOCK_ALIAS.id,
            ELEMENT_SEARCH_RESULT_MOCK_ALIAS.modelId,
          ),
        )
        .reply(HttpStatusCode.Ok, bioEntityFixture);
      mockedAxiosOldClient
        .onGet(apiPath.getReactionsWithIds([Number(bioEntityFixture.id)]))
        .reply(HttpStatusCode.Ok, bioEntityFixture);
      const { store } = getReduxStoreWithActionsListener();
      const { dispatch } = store;

      await handleAliasResults(dispatch, ELEMENT_SEARCH_RESULT_MOCK_ALIAS, {
        ...SEARCH_CONFIG_MOCK,
        isResultDrawerOpen: true,
      })(ELEMENT_SEARCH_RESULT_MOCK_ALIAS);

      await waitFor(() => {
        const actions = store.getActions();
        expect(actions.length).toBeGreaterThan(SIZE_OF_EMPTY_ARRAY);

        const actionTypes = actions.map(action => action.type);

        expect(actionTypes).toEqual([
          'project/getMultiBioEntity/pending',
          'project/getBioEntityById/pending',
          'project/getCommentElement/pending',
          'project/getCommentElement/fulfilled',
          'entityNumber/addNumbersToEntityNumberData',
          'project/getBioEntityById/fulfilled',
          'entityNumber/addNumbersToEntityNumberData',
          'project/getMultiBioEntity/fulfilled',
          'drawer/selectTab',
          'drawer/openBioEntityDrawerById',
        ]);
      });
    });

    describe('fitBounds after search', () => {
      it('should fit bounds after search when hasFitBounds is true', async () => {
        mockedAxiosClient
          .onGet(
            apiPath.getBioEntityContentsStringWithQuery({
              searchQuery: ELEMENT_SEARCH_RESULT_MOCK_ALIAS.id.toString(),
              isPerfectMatch: true,
            }),
          )
          .reply(HttpStatusCode.Ok, bioEntityResponseFixture);
        const { store } = getReduxStoreWithActionsListener();
        const { dispatch } = store;

        await handleAliasResults(dispatch, ELEMENT_SEARCH_RESULT_MOCK_ALIAS, {
          ...SEARCH_CONFIG_MOCK,
          hasFitBounds: true,
        })(ELEMENT_SEARCH_RESULT_MOCK_ALIAS);

        await waitFor(() => {
          expect(searchFitBounds).toHaveBeenCalled();
        });
      });

      it('should not fit bounds after search when hasFitBounds is false', async () => {
        mockedAxiosClient
          .onGet(
            apiPath.getBioEntityContentsStringWithQuery({
              searchQuery: ELEMENT_SEARCH_RESULT_MOCK_ALIAS.id.toString(),
              isPerfectMatch: true,
            }),
          )
          .reply(HttpStatusCode.Ok, bioEntityResponseFixture);
        const { store } = getReduxStoreWithActionsListener();
        const { dispatch } = store;

        await handleAliasResults(dispatch, ELEMENT_SEARCH_RESULT_MOCK_ALIAS, {
          ...SEARCH_CONFIG_MOCK,
          hasFitBounds: false,
        })(ELEMENT_SEARCH_RESULT_MOCK_ALIAS);

        await waitFor(() => {
          expect(searchFitBounds).not.toHaveBeenCalled();
        });
      });
    });
  });
});
